@@ -35,7 +35,26 @@ BEGIN {
    *socket_inet_aton = \&Socket::inet_aton; # take a copy, in case Coro::LWP overrides it
 }
 
+BEGIN {
+   my $af_inet6 = eval { &Socket::AF_INET6 };
+   eval "sub AF_INET6() { $af_inet6 }"; die if $@;
+
+   delete $AnyEvent::PROTOCOL{ipv6} unless $af_inet6;
+}
+
+BEGIN {
+   # broken windows perls use undocumented error codes...
+   if ($^O =~ /mswin32/i) {
+      eval "sub WSAWOULDBLOCK()  { 10035 }";
+      eval "sub WSAEINPROGRESS() { 10036 }";
+   } else {
+      eval "sub WSAWOULDBLOCK()  { -1e99 }"; # should never match any errno value
+      eval "sub WSAEINPROGRESS() { -1e99 }"; # should never match any errno value
+   }
+}
+
 our @EXPORT = qw(fh_nonblocking guard);
+our @EXPORT_OK = qw(AF_INET6 WSAWOULDBLOCK WSAEINPROGRESS);
 
 our $VERSION = '1.0';
 
